<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter\Plugin\Field\FieldWidget;

use Drupal\Core\Config\Entity\ThirdPartySettingsInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'user_selected_formatter_default' widget.
 */
#[FieldWidget(
  id: 'user_selected_formatter_default',
  label: new TranslatableMarkup('User selected formatter'),
  description: new TranslatableMarkup('Allows to select a formatter preset'),
  field_types: ['user_selected_formatter'],
  multiple_values: FALSE,
)]
class UserSelectedFormatterDefault extends WidgetBase {

  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    protected EntityFieldManagerInterface $fieldManager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_field.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $fieldName = $this->fieldDefinition->getSetting('field');
    $entity = $items->getEntity();
    $options = [];
    $definition = $this->fieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle())[$fieldName] ?? NULL;

    if ($definition instanceof ThirdPartySettingsInterface) {
      foreach ($definition->getThirdPartySetting('user_selected_formatter', 'presets', []) as $presetId => $preset) {
        $options[$presetId] = $preset['label'];
      }
    }

    return [
      'value' => [
        '#type' => 'select',
        '#title' => $this->fieldDefinition->getLabel(),
        '#default_value' => $items[$delta]->value ?? NULL,
        '#options' => $options,
        '#required' => TRUE,
        '#empty_option' => $this->t('- Select -'),
      ],
    ];
  }

}
