<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\user_selected_formatter\Model\Preset;

/**
 * User Selected Formatter service.
 */
readonly class UserSelectedFormatterService implements UserSelectedFormatterServiceInterface {

  public function __construct(
    protected EntityFieldManagerInterface $fieldManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getPresetByTargetField(FieldableEntityInterface $entity, FieldDefinitionInterface $targetFieldDefinition): ?Preset {
    $targetFieldName = $targetFieldDefinition->getName();
    if (!$entity->hasField($targetFieldName)) {
      throw new \InvalidArgumentException("The {$entity->getEntityTypeId()}:{$entity->id()} entity doesn't have a '$targetFieldName' field");
    }

    $map = $this->fieldManager->getFieldMapByFieldType('user_selected_formatter');
    $userSelectedFormatterFields = $map[$entity->getEntityTypeId()] ?? NULL;
    if (!$userSelectedFormatterFields) {
      throw new \InvalidArgumentException("The {$entity->getEntityTypeId()} entity type doesn't define a user_selected_formatter field");
    }

    $definitions = $this->fieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach (array_keys($userSelectedFormatterFields) as $userSelectedFormatterFieldName) {
      if (
        $entity->hasField($userSelectedFormatterFieldName) &&
        $definitions[$userSelectedFormatterFieldName]->getSetting('field') === $targetFieldDefinition->getName()
      ) {
        $presetId = $entity->get($userSelectedFormatterFieldName)->first()->getValue()['value'];
        $presets = $targetFieldDefinition->getThirdPartySetting('user_selected_formatter', 'presets', []);
        if (isset($presets[$presetId])) {
          return Preset::createFromArray($presets[$presetId]);
        }
      }
    }

    return NULL;
  }

}
