<?php

declare(strict_types=1);

namespace Drupal\Tests\user_selected_formatter\Kernel;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\FieldConfigInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\user_selected_formatter\Traits\TestFieldCreationTrait;

/**
 * Tests dependencies.
 *
 * @group user_selected_formatter
 */
class DependencyTest extends KernelTestBase {

  use TestFieldCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'node',
    'text',
    'user_selected_formatter',
    'system',
  ];

  /**
   * @covers \Drupal\user_selected_formatter\Plugin\Field\FieldType\UserSelectedFormatterItem::calculateDependencies
   */
  public function testTargetFieldRemoval(): void {
    NodeType::create(['type' => 'page'])->save();
    $formattedField = $this->createFormattedField('body');
    $this->assertInstanceOf(FieldConfigInterface::class, $formattedField);
    $formatterField = $this->createUserSelectedFormatterField('formatter_field', 'body');
    $this->assertInstanceOf(FieldConfigInterface::class, $formatterField);

    // Deleting the target field also deleted the formatter field.
    $formattedField->delete();
    $this->assertNull(FieldConfig::load($formattedField->id()));
    $this->assertNull(FieldConfig::load($formatterField->id()));
  }

}
