<?php

declare(strict_types=1);

namespace Drupal\Tests\user_selected_formatter\Traits;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\field\FieldConfigInterface;

/**
 * Tests reusable code.
 */
trait TestFieldCreationTrait {

  /**
   * Creates a field to be formatted with user selected formatter.
   *
   * @return \Drupal\field\FieldConfigInterface
   *   The field config entity.
   */
  protected function createFormattedField(string $fieldName): FieldConfigInterface {
    FieldStorageConfig::create([
      'entity_type' => 'node',
      'field_name' => $fieldName,
      'type' => 'text_with_summary',
    ])->save();
    $field = FieldConfig::create([
      'entity_type' => 'node',
      'bundle' => 'page',
      'field_name' => $fieldName,
      'label' => ucfirst($fieldName),
    ]);
    $field->save();
    return $field;
  }

  /**
   * Creates a user_selected_formatter field.
   *
   * @return \Drupal\field\FieldConfigInterface
   *   The field config entity.
   */
  protected function createUserSelectedFormatterField(string $fieldName, string $targetFieldName): FieldConfigInterface {
    FieldStorageConfig::create([
      'entity_type' => 'node',
      'field_name' => $fieldName,
      'type' => 'user_selected_formatter',
    ])->save();
    $field = FieldConfig::create([
      'entity_type' => 'node',
      'bundle' => 'page',
      'field_name' => $fieldName,
      'settings' => [
        'field' => $targetFieldName,
      ],
    ]);
    $field->save();
    return $field;
  }

}
