<?php

namespace Drupal\user_selected_formatter_ui\Form;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\field\FieldConfigInterface;
use Drupal\user_selected_formatter_ui\UserSelectionFormatterUiTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Preset deletion form.
 */
class UserSelectedFormatterUiPresetDeleteForm extends ConfirmFormBase {

  use UserSelectionFormatterUiTrait;

  /**
   * Static cache for field config entity.
   */
  protected FieldConfigInterface $fieldConfig;

  /**
   * Static cache for preset data.
   */
  protected array $preset;

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static($container->get('entity_type.manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): MarkupInterface {
    return $this->t('Delete preset %preset from %field field?', [
      '%preset' => $this->getPreset()['label'],
      '%field' => $this->getFieldConfig()->getLabel(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return $this->getListUrl($this->getFieldConfig());
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'user_selected_formatter_preset_delete';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $presets = $this->getFieldConfig()->getThirdPartySetting('user_selected_formatter', 'presets', []);
    unset($presets[$this->getPreset()['id']]);
    if (!empty($presets)) {
      $this->getFieldConfig()->setThirdPartySetting('user_selected_formatter', 'presets', $presets);
    }
    else {
      $this->getFieldConfig()->unsetThirdPartySetting('user_selected_formatter', 'presets');
    }
    $this->getFieldConfig()->save();
    $form_state->setRedirectUrl($this->getListUrl($this->getFieldConfig()));
    $this->messenger()->addStatus($this->t('Preset %preset has been deleted from field %field', [
      '%preset' => $this->getPreset()['label'],
      '%field' => $this->getFieldConfig()->getLabel(),
    ]));
  }

  /**
   * Returns the preset data.
   *
   * @return array
   *   The preset data.
   */
  protected function getPreset(): array {
    if (!isset($this->preset)) {
      $presetId = $this->getRouteMatch()->getParameter('preset');
      $this->preset = $this->getFieldConfig()->getThirdPartySetting('user_selected_formatter', 'presets', [])[$presetId];
    }
    return $this->preset;
  }

  /**
   * Returns the field config entity.
   *
   * @return \Drupal\field\FieldConfigInterface
   *   The field config entity.
   */
  protected function getFieldConfig(): FieldConfigInterface {
    if (!isset($this->fieldConfig)) {
      $this->fieldConfig = $this->getRouteMatch()->getParameter('field_config');
    }
    return $this->fieldConfig;
  }

}
