<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter_ui\ParamConverter;

use Drupal\Core\ParamConverter\ParamConverterInterface;
use Drupal\field\FieldConfigInterface;
use Symfony\Component\Routing\Route;

/**
 * Resolves parameters of type 'user_selected_formatter_preset'.
 */
class UserSelectionFormatterUiParamConverter implements ParamConverterInterface {

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults): ?string {
    if (!$value || !$defaults['field_config'] instanceof FieldConfigInterface) {
      return NULL;
    }
    if ($defaults['field_config']->getType() === 'user_selected_formatter') {
      // The user_selected_formatter field type cannot have presets.
      return NULL;
    }

    $presets = $defaults['field_config']->getThirdPartySetting('user_selected_formatter', 'presets', []);
    if (!isset($presets[$value])) {
      return NULL;
    }

    // @todo Convert to Preset instance.
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route): bool {
    return !empty($definition['type']) && $definition['type'] === 'user_selected_formatter_preset';
  }

}
