<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter_ui\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Creates "Add preset" actions for each field config.
 */
class UserSelectionFormatterUiLocalAction extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id): static {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition): array {
    $this->derivatives = [];

    foreach ($this->entityTypeManager->getDefinitions() as $entityTypeId => $entityType) {
      if ($entityType->get('field_ui_base_route')) {
        $this->derivatives["user_selected_formatter_preset.$entityTypeId"] = [
          'route_name' => "entity.field_config.$entityTypeId.user_selected_formatter_preset.add",
          'title' => $this->t('Add preset'),
          'appears_on' => [
            "entity.field_config.$entityTypeId.user_selected_formatter_preset",
          ],
        ];
      }
    }

    return $this->derivatives;
  }

}
