<?php

declare(strict_types=1);

namespace Drupal\user_selected_formatter_ui;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\field\FieldConfigInterface;

/**
 * Reusable code.
 */
trait UserSelectionFormatterUiTrait {

  /**
   * Returns the presets list URL object given a field config entity.
   *
   * @param \Drupal\field\FieldConfigInterface $fieldConfig
   *   The field config entity.
   *
   * @return \Drupal\Core\Url
   *   The presets list URL object.
   */
  protected function getListUrl(FieldConfigInterface $fieldConfig): Url {
    \assert(property_exists($this, 'entityTypeManager') && $this->entityTypeManager instanceof EntityTypeManagerInterface);
    $targetEntityTypeId = $fieldConfig->getTargetEntityTypeId();
    $bundleEntityTypeId = $this->entityTypeManager->getDefinition($targetEntityTypeId)->getBundleEntityType();
    return Url::fromRoute("entity.field_config.$targetEntityTypeId.user_selected_formatter_preset", [
      $bundleEntityTypeId => $fieldConfig->getTargetBundle(),
      'field_config' => $fieldConfig->id(),
    ]);
  }

}
