<?php

declare(strict_types=1);

namespace Drupal\Tests\user_selected_formatter_ui\Kernel;

use Drupal\Component\Utility\Html;
use Drupal\KernelTests\KernelTestBase;
use Drupal\node\Entity\NodeType;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\Tests\user_selected_formatter\Traits\TestFieldCreationTrait;
use mglaman\DrupalTestHelpers\RequestTrait;
use Symfony\Component\HttpFoundation\Request;

/**
 * Test UIs.
 *
 * @group user_selected_formatter
 */
class UiTest extends KernelTestBase {

  use RequestTrait;
  use TestFieldCreationTrait;
  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'field',
    'user_selected_formatter',
    'user_selected_formatter_ui',
    'text',
    'system',
    'field_ui',
    'user',
    'filter',
  ];

  /**
   * @covers \Drupal\user_selected_formatter\Plugin\Field\FieldType\UserSelectedFormatterItem::fieldSettingsForm
   */
  public function testFieldSettingsOptions(): void {
    $this->installEntitySchema('user');
    $account = $this->createUser(['administer node fields']);
    $this->setCurrentUser($account);
    NodeType::create(['type' => 'page', 'name' => 'Page'])->save();

    $this->createFormattedField('body0');
    $this->createFormattedField('body1');
    $this->createFormattedField('body2');
    $this->createUserSelectedFormatterField('field0', 'body0');
    $this->createUserSelectedFormatterField('field1', 'body1');
    $this->createUserSelectedFormatterField('field2', 'body2');

    $this->assertAvailableFieldOptions(0);
    $this->assertAvailableFieldOptions(1);
    $this->assertAvailableFieldOptions(2);
  }

  /**
   * Asserts the target fields available for a given formatter field.
   *
   * @param int $delta
   *   The index of the field.
   */
  protected function assertAvailableFieldOptions(int $delta): void {
    $this->doRequest(Request::create("/admin/structure/types/manage/page/fields/node.page.field$delta"));
    // The field is able to target $targetFieldName.
    $this->assertOption('edit-settings-field', "body$delta", "Option edit-settings-field:body$delta doesn't exist");
    // But not fields assigned to different user_selected_formatter fields.
    for ($i = 0; $i < 3; $i++) {
      if ($i !== $delta) {
        $this->assertNoOption('edit-settings-field', "body$i", "Option edit-settings-field:body$i exists");
      }
    }

    // The field cannot target user_selected_formatter type field.
    for ($i = 0; $i < 3; $i++) {
      $this->assertNoOption('edit-settings-field', "field$i", "Option edit-settings-field:field$i exists");
    }

    // Make sure the field ID is the same for all test cases.
    Html::resetSeenIds();
  }

}
